@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-/* $Id: acconfig.h,v 1.4 2001/03/07 10:42:10 gray Exp $ */
+/* $Id: acconfig.h,v 1.6 2001/04/26 11:25:00 gray Exp $ */
 
 #undef PW_AUTH_UDP_PORT
 #undef PW_ACCT_UDP_PORT
@@ -57,6 +57,8 @@
 #undef u_long
 /* ************************************************************************** */
 
+#undef HAVE_GETOPT_LONG
+
 #undef HAVE_SYS_ERRLIST
 
 #undef HAVE_LIBDBM
@@ -110,7 +112,7 @@
 
 #undef YACC_DEBUG
 
-#if defined(sun)
+#if defined(sun) && !defined(__EXTENSIONS__)
 # define __EXTENSIONS__
 #endif
 
