@@ -17,7 +17,7 @@
  *
  */
 
-/* $Id: log.h,v 1.2 2000/12/29 11:48:09 gray Exp $ */
+/* $Id: log.h,v 1.5 2001/03/02 16:54:01 gray Exp $ */
 
 #ifndef __log_h
 #define __log_h
@@ -54,6 +54,10 @@
 
 #define RLOG_DEFAULT            (RLOG_AUTH | RLOG_FAILED_PASS)
 
+int radvsprintf(/*char *string, size_t size, char *fmt, va_list ap*/);
+int radsprintf(/*char *string, size_t size, char *fmt, va_alist*/);
+int radfprintf(/*FILE *file, char *fmt, va_alist*/);
+
 #define MKSTRING(x) #x 
 #define insist(cond) \
  ((void) ((cond) || __insist_failure(MKSTRING(cond), __FILE__, __LINE__)))
@@ -62,7 +66,10 @@
 	
 #define RADIUS_DEBUG_BUFFER_SIZE 1024
 
-typedef struct channel {
+typedef struct chanlist Chanlist;
+typedef struct channel Channel;
+
+struct channel {
 	struct channel *next;
 	int ucnt;
 	char *name;
@@ -72,8 +79,13 @@ typedef struct channel {
 		char *file;
 	} id;
 	int options;
-} Channel;
- 
+};
+
+struct chanlist {
+	struct chanlist *next;
+	Channel *channel;
+};
+
 /* log.c */
 extern int debug_level[];
 extern int log_mode;
@@ -81,7 +93,7 @@ extern int log_mode;
 void            initlog(char*);
 int		radlog(/*int, char *, ...*/);
 char           *debug_sprintf(/*char *, ...*/);
-int             debug_output(char *, int, char *, char *);
+void            debug_output(char *, int, char *, char *);
 int             __insist_failure(char *, char *, int);
 
 #ifndef MAX_DEBUG_LEVEL
@@ -109,11 +121,13 @@ void log_init();
 void log_cleanup();
 Channel *register_channel(Channel *);
 Channel *install_channel(char *name, int mode, int prio, char *file, int opt);
-void register_category(int level, Channel *channel);
+void register_category(int cat, Chanlist *chanlist);
+Chanlist *make_chanlist(Channel *chan);
+
 Channel *channel_lookup(char *name);
 
-int set_debug_levels(char *str);
+void set_debug_levels(char *str);
 int set_module_debug_level(char *name, int level);
-int clear_debug();
+void clear_debug();
 
 #endif
