@@ -32,9 +32,8 @@ typedef unsigned long	UINT4;
 
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
-#else
-# include <string.h>
 #endif
+#include <string.h>
 
 #ifndef HAVE_BZERO
 # define bzero(s,n) memset(s, 0, n)
@@ -44,8 +43,11 @@ typedef unsigned long	UINT4;
 extern int strncasecmp(char*, char*, int);
 #endif
 
-#ifdef TM_IN_SYS_TIME
+#ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
+# ifdef TIME_WITH_SYS_TIME
+#  include <time.h>
+# endif
 #else
 # include <time.h>
 #endif
@@ -92,4 +94,11 @@ extern int strncasecmp(char*, char*, int);
 #  define ut_user ut_name
 #endif
 
+#if defined (sun) && defined(__svr4__)
+RETSIGTYPE (*sun_signal(int signo, void (*func)(int)))(int);
+#define signal sun_signal
+#endif
+int set_nonblocking(int fd);
+int getmaxfd();
+
 #endif /* SYSDEP_H_INCLUDED */
