@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,11 +16,11 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#define RADIUS_MODULE 3
+#define RADIUS_MODULE 5
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: acct.c,v 1.4 2001/01/26 11:48:12 gray Exp $";
+"@(#) $Id: acct.c,v 1.10 2001/03/12 11:18:21 gray Exp $";
 #endif
 #define LOG_EMPTY_USERNAME
 
@@ -44,9 +44,7 @@ static char rcsid[] =
 #include <signal.h>
 #include <errno.h>
 #include <sys/wait.h>
-#ifdef sun
-# include <fcntl.h>
-#endif
+#include <fcntl.h>
 #include <sysdep.h>
 #include <radiusd.h>
 #include <radutmp.h>
@@ -57,7 +55,7 @@ static char rcsid[] =
 int	doradwtmp = 1;
 
 static char porttypes[] = "ASITX";
-static int write_wtmp(struct radutmp *ut, int status);
+static int write_wtmp(struct radutmp *ut);
 static int write_nas_restart(int status, UINT4 addr);
 static int check_ts(struct radutmp *ut);
 
@@ -113,7 +111,7 @@ radzap(nasaddr, port, user, t)
 			/*
 			 *	Add a logout entry to the wtmp file.
 			 */
-			write_wtmp(&u, DV_ACCT_STATUS_TYPE_STOP);
+			write_wtmp(&u);
 		}
 		close(fd);
 	}
@@ -178,11 +176,9 @@ decode_phone_number(buf, size, nstr)
 }
 
 
-
 int
-write_wtmp(ut, status)
+write_wtmp(ut)
 	struct radutmp *ut;
-	int status;
 {
 	FILE *fp;
 		
@@ -197,7 +193,7 @@ write_wtmp(ut, status)
 }
 
 void
-backslash(dst, src, len)
+backslashify(dst, src, len)
 	char *dst;
 	char *src;
 	int len;
@@ -244,9 +240,6 @@ end:
 	*dst = 0;
 }
 
-#if 0
-/* FIXME: still not needed */
-   
 int
 check_attribute(check_pairs, pair_attr, pair_value, def)
 	VALUE_PAIR *check_pairs;
@@ -256,14 +249,20 @@ check_attribute(check_pairs, pair_attr, pair_value, def)
 {
 	VALUE_PAIR *pair;
 
-	pair = check_pairs;
-	while (check_pairs && pair = pairfind(check_pairs, pair_attr)) {
+	if ((pair = pairfind(check_pairs, pair_attr)) == NULL)
+		return def;
+	do {
 		if (pair->lvalue == pair_value)
 			return 1;
 		check_pairs = pair->next;
-	}
-	return def;
+	} while (check_pairs && (pair = pairfind(check_pairs, pair_attr)));
+	return 0;
 }
+
+#ifdef DA_ACCT_TYPE
+# define ACCT_TYPE(req,t) check_attribute(req, DA_ACCT_TYPE, t, 1)
+#else
+# define ACCT_TYPE(req,t) 1
 #endif
 
 /*
@@ -279,7 +278,6 @@ rad_accounting_new(authreq, dowtmp)
 	int		rb_record = 0;
 	int		status = -1;
 	int		nas_address = 0;
-	int		framed_address = 0;
 	int		protocol = -1;
 	time_t		t;
 	int		fd;
@@ -287,7 +285,6 @@ rad_accounting_new(authreq, dowtmp)
 	int		just_an_update = 0;
 	int		port_seen = 0;
 	int		nas_port_type = 0;
-	char           *called_id = NULL;
 	
 	/*
 	 *	Which type is this.
@@ -351,20 +348,25 @@ rad_accounting_new(authreq, dowtmp)
 	memset(&ut, 0, sizeof(ut));
 	ut.porttype = 'A';
 
+	if (authreq->realm) {
+		REALM *realm = realm_find(authreq->realm);
+		if (realm)
+			ut.realm_address = realm->ipaddr;
+	}
+	
 	/*
 	 *	First, find the interesting attributes.
 	 */
 	for (vp = authreq->request; vp; vp = vp->next) {
 		switch (vp->attribute) {
 		case DA_USER_NAME:
-			backslash(ut.login, vp->strvalue, RUT_NAMESIZE);
+			backslashify(ut.login, vp->strvalue, RUT_NAMESIZE);
 			break;
 		case DA_ORIG_USER_NAME:
-			backslash(ut.orig_login, vp->strvalue, RUT_NAMESIZE);
+			backslashify(ut.orig_login, vp->strvalue, RUT_NAMESIZE);
 			break;
 		case DA_LOGIN_IP_HOST:
 		case DA_FRAMED_IP_ADDRESS:
-			framed_address = vp->lvalue;
 			ut.framed_address = htonl(vp->lvalue);
 			break;
 		case DA_FRAMED_PROTOCOL:
@@ -388,7 +390,6 @@ rad_accounting_new(authreq, dowtmp)
 					 vp->strlength);
 			break;
 		case DA_CALLED_STATION_ID:
-			called_id = vp->strvalue;
 			break;
 		case DA_ACCT_SESSION_ID:
 			store_session_id(ut.session_id,
@@ -478,7 +479,7 @@ rad_accounting_new(authreq, dowtmp)
 
 #ifdef USE_NOTIFY
 	if (!port_seen) {
-		notify_acct(ut.login, status, called_id);
+		notify_acct(ut.login, status);
 	}
 #endif
 	
@@ -495,6 +496,9 @@ rad_accounting_new(authreq, dowtmp)
 	    nas_port_type == DV_NAS_PORT_TYPE_SYNC)
 		return 0;
 
+	if (!ACCT_TYPE(authreq->request, DV_ACCT_TYPE_SYSTEM))
+		return 0;
+	
 	/*
 	 *	Enter into the radutmp file.
 	 */
@@ -625,7 +629,7 @@ rad_accounting_new(authreq, dowtmp)
 	 */
 	if (dowtmp) {
 		stat_update(&ut, status);
-		write_wtmp(&ut, status);
+		write_wtmp(&ut);
 	} else if (just_an_update) {
 		stat_update(&ut, status);
 	} else {
@@ -675,7 +679,7 @@ write_detail(authreq, authtype, f)
 	VALUE_PAIR	*pair;
 	UINT4		nas;
 	NAS		*cl;
-	long		curtime;
+	time_t		curtime;
 	int		ret = 0;
 	struct stat	st;
 
@@ -730,7 +734,7 @@ write_detail(authreq, authtype, f)
 	} else {
 
 		/* Post a timestamp */
-		fprintf(outfd, ctime(&curtime));
+		fprintf(outfd, "%s", ctime(&curtime));
 
 		/* Decide which username to log */
 		if (!strip_names) {
@@ -744,9 +748,7 @@ write_detail(authreq, authtype, f)
 			else
 				pair = pairfind(authreq->request, DA_USER_NAME);
 			if (pair) {
-				fprintf(outfd, "\t");
-				fprint_attr_val(outfd, pair);
-				fprintf(outfd, "\n");
+				radfprintf(outfd, "\t%A\n", pair);
 			}
 		}
 
@@ -761,9 +763,7 @@ write_detail(authreq, authtype, f)
 				if (!strip_names)
 					break;
 			default:
-				fprintf(outfd, "\t");
-				fprint_attr_val(outfd, pair);
-				fprintf(outfd, "\n");
+				radfprintf(outfd, "\t%A\n", pair);
 			} 
 			pair = pair->next;
 		}
@@ -795,18 +795,18 @@ write_detail(authreq, authtype, f)
 }
 
 int
-rad_accounting_orig(authreq, authtype, f)
+rad_accounting_orig(authreq, authtype)
 	AUTH_REQ *authreq;
 	int authtype;
-	char *f;
 {
-	int rc;
+	int rc = 0;
 
-	rc = write_detail(authreq, authtype, "detail");
+	if (ACCT_TYPE(authreq->request, DV_ACCT_TYPE_DETAIL))
+		rc = write_detail(authreq, authtype, "detail");
 #ifdef USE_SQL
-	rad_sql_acct(authreq);
+	if (ACCT_TYPE(authreq->request, DV_ACCT_TYPE_SQL))
+		rad_sql_acct(authreq);
 #endif
-	
 	return rc;
 }
 
@@ -820,7 +820,7 @@ rad_accounting(authreq, activefd)
 	int activefd;
 {
 	int auth;
-	char pw_digest[AUTH_PASS_LEN];
+	u_char pw_digest[AUTH_PASS_LEN];
 
 	/*
 	 *	See if we know this client, then check the
@@ -847,7 +847,7 @@ rad_accounting(authreq, activefd)
 #endif
 	
 	if (rad_accounting_new(authreq, doradwtmp) == 0 &&
-	    rad_accounting_orig(authreq, auth, NULL) == 0) {
+	    rad_accounting_orig(authreq, auth) == 0) {
 		/*
 		 *	Now send back an ACK to the NAS.
 		 */
@@ -860,6 +860,7 @@ rad_accounting(authreq, activefd)
 	return -1;
 }
 
+/*ARGSUSED*/
 void
 rad_acct_xmit(type, code, data, fd)
 	int type;
@@ -877,22 +878,12 @@ rad_acct_xmit(type, code, data, fd)
 		       req->id);
 	} else {
 		radlog(L_NOTICE,
-		       _("Dropping ACCT packet: client %s, ID: %d"),
+		       _("Dropping ACCT packet: client %s, ID: %d: duplicate packet"),
 		       client_name(req->ipaddr),
 		       req->id);
 	}
 }
 
-/*
- *	Timeout handler (10 secs)
- */
-static int got_alrm;
-static void
-alrm_handler()
-{
-	got_alrm = 1;
-}
-
 /*
  *	Check one terminal server to see if a user is logged in.
  * Return value:
@@ -933,13 +924,7 @@ static int
 check_ts(ut)
 	struct radutmp *ut;
 {
-	int	pid, st, e;
-	int	n;
 	NAS	*nas;
-	char	address[DOTTED_QUAD_LEN];
-	char	port[32];
-	char	session_id[RUT_IDSIZE+1];
-	RETSIGTYPE (*handler)(int);
 
 	/*
 	 *	Find NAS type.
@@ -950,75 +935,78 @@ check_ts(ut)
 	}
 
 	/*
-	 *	Fork.
+	 * Handle two special types
 	 */
-	handler = signal(SIGCHLD, SIG_DFL);
-	if ((pid = fork()) < 0) {
-		radlog(L_ERR|L_PERROR, _("check_ts(): can't fork"));
-		signal(SIGCHLD, handler);
-		return -1;
-	}
+	if (strcmp(nas->nastype, "true") == 0)
+		return 1;
+	else if (strcmp(nas->nastype, "false") == 0)
+		return 0;
 
-	if (pid > 0) {
-		/*
-		 *	Parent - Wait for checkrad to terminate.
-		 *	We timeout in 10 seconds.
-		 */
-		got_alrm = 0;
-		signal(SIGALRM, alrm_handler);
-		alarm(10);
-		while ((e = waitpid(pid, &st, 0)) != pid)
-			if (e < 0 && (errno != EINTR || got_alrm))
-				break;
-		alarm(0);
-		signal(SIGCHLD, handler);
-		if (got_alrm) {
-			kill(pid, SIGTERM);
-			sleep(1);
-			kill(pid, SIGKILL);
-			radlog(L_WARN, _("check_ts(): timeout waiting for checkrad"));
-			return -1;
-		}
-		if (e < 0) {
-			radlog(L_ERR|L_PERROR, 
-				_("check_ts(): error in waitpid()"));
-			return -1;
-		}
-		debug(10,
-			("got response %d", WEXITSTATUS(st))); 
-		return WEXITSTATUS(st);
-	}
-
-	/*
-	 *	Child - exec checkrad with the right parameters.
-	 */
-	for (n = 32; n >= 3; n--)
-		close(n);
-
-	ipaddr2str(address, ntohl(ut->nas_address));
-	sprintf(port, "%d", ut->nas_port);
-	strncpy(session_id, ut->session_id, RUT_IDSIZE);
-	session_id[RUT_IDSIZE] = 0;
-
-	debug(10,
-		("starting checkrad -d %s -t %s -h %s -p %s -u %s -s %s",
-		radius_dir,
-	    	nas->nastype, address, port, ut->login, session_id));
-	execl(CHECKRAD, "checkrad", 
-		"-d", radius_dir,	
-		"-t", nas->nastype, 
-		"-h", address, 
-		"-p", port,
-		"-u", ut->orig_login, 
-		"-s", session_id, NULL);
-	radlog(L_ERR, "check_ts(): exec %s: %s", CHECKRAD, strerror(errno));
-
-	/*
-	 *	Exit - 2 means "some error occured".
-	 */
-	exit(2);
+	return checkrad(nas, ut);
 }
 
+int
+rad_check_realm(realmname)
+	char *realmname;
+{
+	REALM *realm;
+	int count;
+	int fd;
+	struct radutmp	u;
+	
+	realm = realm_find(realmname);
+	if (!realm || realm->maxlogins == 0)
+		return 0;
+
+	if ((fd = open(radutmp_path, O_CREAT|O_RDWR, 0644)) < 0)
+		return 0;
+
+	/*
+	 * Pass I: scan radutmp file
+	 */
+	count = 0;
+	while (read(fd, &u, sizeof(u)) == sizeof(u))
+		if (u.realm_address == realm->ipaddr && u.type == P_LOGIN) 
+			count++;
+
+	if (count < realm->maxlogins) {
+		close(fd);
+		return 0;
+	}
+
+	/*
+	 * Pass II: verify logins
+	 */
+	lseek(fd, (off_t)0, SEEK_SET);
+	count = 0;
+
+	while (read(fd, &u, sizeof(u)) == sizeof(u)) {
+		if (!(u.realm_address == realm->ipaddr && u.type == P_LOGIN)) 
+			continue;
+
+		if (rad_check_ts(&u) == 1) {
+			count++;
+		} else {
+			/*
+			 *	False record - zap it.
+			 */
+			u.type = P_IDLE;
+			u.time = time(NULL);
+				
+			lseek(fd, -(off_t)sizeof(u), SEEK_CUR);
+
+                        /* lock the file while writing. */
+			rad_lock(fd, sizeof(u), 0, SEEK_CUR);
+			write(fd, &u, sizeof(u));
+			rad_unlock(fd, sizeof(u),
+				   -(off_t)sizeof(u), SEEK_CUR);
+
+			write_wtmp(&u);
+		}
+	}
+	close(fd);
+	return !(count < realm->maxlogins);
+}
 
 /*
  *	See if a user is already logged in.
@@ -1037,7 +1025,7 @@ rad_check_multi(name, request, maxsimul)
 {
 	int		fd;
 	int		count;
-	struct radutmp	u, empty;
+	struct radutmp	u;
 	VALUE_PAIR	*fra;
 	int		mpp = 1;
 	UINT4		ipno = 0;
@@ -1049,7 +1037,7 @@ rad_check_multi(name, request, maxsimul)
 	 *	We don't lock in the first pass.
 	 */
 	count = 0;
-	while(read(fd, &u, sizeof(u)) == sizeof(u))
+	while (read(fd, &u, sizeof(u)) == sizeof(u))
 		if (strncmp(name, u.login, RUT_NAMESIZE) == 0
 		    && u.type == P_LOGIN) {
 			count++;
@@ -1066,8 +1054,6 @@ rad_check_multi(name, request, maxsimul)
 	 */
 	if ((fra = pairfind(request, DA_FRAMED_IP_ADDRESS)) != NULL)
 		ipno = htonl(fra->lvalue);
-	memset(&empty, 0, sizeof(empty));
-
 
 	/*
 	 *	Allright, there are too many concurrent logins.
@@ -1092,15 +1078,18 @@ rad_check_multi(name, request, maxsimul)
 				 *	False record - zap it.
 				 */
 
+				u.type = P_IDLE;
+				u.time = time(NULL);
+				
 				lseek(fd, -(off_t)sizeof(u), SEEK_CUR);
 
                                 /* lock the file while writing. */
-	                        rad_lock(fd, sizeof(empty), 0, SEEK_CUR);
-                                write(fd, &empty, sizeof(empty));
-                                rad_unlock(fd, sizeof(empty),
-					   -(off_t)sizeof(empty), SEEK_CUR);
+	                        rad_lock(fd, sizeof(u), 0, SEEK_CUR);
+                                write(fd, &u, sizeof(u));
+                                rad_unlock(fd, sizeof(u),
+					   -(off_t)sizeof(u), SEEK_CUR);
 
-				write_wtmp(&u, DV_ACCT_STATUS_TYPE_STOP);
+				write_wtmp(&u);
 			}
 		}
 	}
@@ -1123,7 +1112,7 @@ write_nas_restart(status, addr)
 		ut.type = P_NAS_SHUTDOWN;
 	ut.nas_address = addr;
 	time(&ut.time);
-	return write_wtmp(&ut, status);
+	return write_wtmp(&ut);
 }
 
 
