@@ -23,7 +23,7 @@
 #define RADIUS_MODULE 8
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: files.c,v 1.12 2001/03/12 11:22:12 gray Exp $";
+"@(#) $Id: files.c,v 1.14 2001/03/30 22:05:19 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -1297,7 +1297,7 @@ read_naslist_entry(unused, fc, fv, file, lineno)
 	memcpy(nasp, &nas, sizeof(nas));
 	
 #ifdef USE_SNMP
-	snmp_attach_nas_stat(nasp, master_process());
+	snmp_attach_nas_stat(nasp);
 #endif
 	nasp->next = naslist;
 	naslist = nasp;
@@ -1881,6 +1881,9 @@ static int server_check_items[] = {
 #endif	
 #ifdef DA_ACCT_TYPE
 	DA_ACCT_TYPE,
+#endif
+#ifdef DA_LOG_MODE_MASK
+	DA_LOG_MODE_MASK,
 #endif
 	DA_MENU,
 	DA_TERMINATION_MENU,
