@@ -1,5 +1,5 @@
 /* This file is part of GNU RADIUS.
- * Copyright (C) 2000, Sergey Poznyakoff
+ * Copyright (C) 2000,2001, Sergey Poznyakoff
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: menu.c,v 1.3 2001/01/22 12:33:38 gray Exp $";
+"@(#) $Id: menu.c,v 1.6 2001/03/12 11:49:37 gray Exp $";
 #endif
 
 static VALUE_PAIR * menu_pairs(char *menu_name, char  *menu_selection);
@@ -40,7 +40,7 @@ void
 process_menu(authreq, activefd, pw_digest)
 	AUTH_REQ        *authreq;
 	int             activefd;
-	char            *pw_digest;
+	u_char          *pw_digest;
 {
         VALUE_PAIR *pair, *term_pair, *new_pair;
 	char menu_name[MAX_MENU_NAME];
@@ -76,7 +76,8 @@ process_menu(authreq, activefd, pw_digest)
 	
 	if ((term_pair = pairfind(pair, DA_TERMINATION_MENU)) != NULL) {
 		/* Change this to a menu state */
-		sprintf(state_value, "MENU=%s", term_pair->strvalue);
+		radsprintf(state_value, sizeof(state_value),
+				"MENU=%s", term_pair->strvalue);
 		term_pair->attribute = DA_STATE;
 		replace_string(&term_pair->strvalue, state_value);
 		term_pair->strlength = strlen(state_value);
@@ -99,7 +100,8 @@ process_menu(authreq, activefd, pw_digest)
 	} else if (pair) {
 		if (new_pair = pairfind(pair, DA_MENU)) {
 			msg = get_menu(new_pair->strvalue);
-			sprintf(state_value, "MENU=%s", new_pair->strvalue);
+			radsprintf(state_value, sizeof(state_value),
+					"MENU=%s", new_pair->strvalue);
 			send_challenge(authreq, msg, state_value, activefd);
 		} else {
 			rad_send_reply(PW_AUTHENTICATION_ACK, authreq,
@@ -178,7 +180,6 @@ menu_pairs(menu_name, menu_selection)
 	char    selection[MAX_MENU_INPUT];
 	int     mode;
 	char    *ptr, *errp;
-	int     nread;
 	VALUE_PAIR      *reply_first;
 	int line_num;
 	
