@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: radiusd.c,v 1.13 2001/03/12 13:57:05 gray Exp $";
+"@(#) $Id: radiusd.c,v 1.17 2001/04/06 11:17:51 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +43,7 @@ static char rcsid[] =
 #include <signal.h>
 #include <errno.h>
 #include <sys/wait.h>
-#if defined(HAVE_GETOPT_LONG)
+#if defined(HAVE_GETOPT_H)
 # include <getopt.h>
 #endif
 #include <radiusd.h>
@@ -53,10 +53,6 @@ static char rcsid[] =
 #include <radutmp.h>
 #include <rewrite.h>
 
-#if defined (sun) && defined(__svr4__)
-RETSIGTYPE (*sun_signal(int signo, void (*func)(int)))(int);
-#define signal sun_signal
-#endif
 
 /* ********************** Request list handling **************************** */
 	
@@ -428,7 +424,7 @@ main(argc, argv)
 	else
 		signal(SIGINT, sig_fatal);
 	
-	for (t = 32; t >= 3; t--)
+	for (t = getmaxfd(); t >= 3; t--)
 		close(t);
 
 	/* 
@@ -449,6 +445,7 @@ main(argc, argv)
 	else
 		acct_port = ntohs(svp->s_port);
 	
+	snmp_init(0, 0, emalloc, efree);
 	/*
 	 *	Read config files.
 	 */
@@ -932,8 +929,17 @@ request_xmit(type, code, data, fd)
 {
 	if (request_class[type].xmit) 
 		request_class[type].xmit(type, code, data, fd);
-	else
+	else 
 		request_class[type].drop(type, data, _("duplicate request"));
+
+	switch (type) {
+	case R_AUTH:
+		stat_inc(auth, ((AUTH_REQ*)data)->ipaddr, num_dup_req);
+		break;
+	case R_ACCT:
+		stat_inc(acct, ((AUTH_REQ*)data)->ipaddr, num_dup_req);
+	}
+
 	request_class[type].free(data);
 }
 
@@ -1345,32 +1351,6 @@ rad_req_drop(type, authreq, status_str)
 }
 /* ************************************************************************* */
 
-#if defined (sun) && defined(__svr4__)
-/*
- *	The signal() function in Solaris 2.5.1 sets SA_NODEFER in
- *	sa_flags, which causes grief if signal() is called in the
- *	handler before the cause of the signal has been cleared.
- *	(Infinite recursion).
- */
-RETSIGTYPE
-(*sun_signal(signo, func))(int)
-	int signo;
-	void (*func)(int);
-{
-	struct sigaction act, oact;
-
-	act.sa_handler = func;
-	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-#ifdef  SA_INTERRUPT		/* SunOS */
-	act.sa_flags |= SA_INTERRUPT;
-#endif
-	if (sigaction(signo, &act, &oact) < 0)
-		return SIG_ERR;
-	return oact.sa_handler;
-}
-#endif
-
 /*ARGSUSED*/
 RETSIGTYPE
 sig_cleanup(sig)
@@ -1674,37 +1654,6 @@ sig_dumpdb(sig)
 	dump_users_db();
 }
 
-int
-master_process()
-{
-	return radius_pid == 0 || getpid() == radius_pid;
-}
-
-#if defined(O_NONBLOCK)
-# define FCNTL_NONBLOCK O_NONBLOCK
-#elif defined(O_NDELAY)
-# define FCNTL_NONBLOCK O_NDELAY
-#else
-# error "Neither O_NONBLOCK nor O_NDELAY are defined"
-#endif
-
-int
-set_nonblocking(fd)
-	int fd;
-{
-	int flags;
-
-	if ((flags = fcntl(fd, F_GETFL, 0)) < 0) {
-		radlog(L_ERR, "F_GETFL: %s", strerror(errno));
-		return -1;
-	}
-	if (fcntl(fd, F_SETFL, flags | FCNTL_NONBLOCK) < 0) {
-		radlog(L_ERR, "F_GETFL: %s", strerror(errno));
-		return -1;
-	}
-	return 0;
-}
-
 int
 open_socket(port, type)
 	int port;
@@ -1754,8 +1703,8 @@ test_shell()
 	struct radutmp ut;
 	Datatype type;
 	Datum datum;
-	
-#define nextkn() if ((tok = strtok(NULL, " \t")) == NULL) {\
+#define opttkn() (tok = strtok(NULL, " \t"))
+#define nextkn() if (opttkn() == NULL) {\
  printf("arg count\n");\
  continue; }  
 #define chktkn() if (!tok) { \
@@ -1778,12 +1727,12 @@ test_shell()
 			continue;
 		case 'h':
 		case '?':
-			printf("h,?                     help\n");
-			printf("q,<EOF>                 quit\n");
-			printf("c LOGIN SID PORT NAS    checkrad\n");
-			printf("r FUNCALL               function call\n");
-			printf("d LEVEL[,LEVEL]         set debug level\n");
-			printf("m                       display memory usage\n"); 
+			printf("h,?                       help\n");
+			printf("q,<EOF>                   quit\n");
+			printf("c NAS LOGIN SID PORT [IP] checkrad\n");
+			printf("r FUNCALL                 function call\n");
+			printf("d LEVEL[,LEVEL]           set debug level\n");
+			printf("m                         display memory usage\n"); 
 			break;
 		case 'd':
 			chktkn();
@@ -1793,18 +1742,21 @@ test_shell()
 			return 0;
 		case 'c': /* checkrad */
 			chktkn();
-			strncpy(ut.orig_login, tok, sizeof(ut.orig_login));
-			nextkn();
-			strncpy(ut.session_id, tok, sizeof(ut.session_id));
-			nextkn();
-			ut.nas_port = atoi(tok);
-			nextkn();
 			nas = nas_by_name(tok);
 			if (!nas) {
 				printf("bad nas\n");
 				continue;
 			}
 			ut.nas_address = nas->ipaddr;
+
+			nextkn();
+			strncpy(ut.orig_login, tok, sizeof(ut.orig_login));
+			nextkn();
+			strncpy(ut.session_id, tok, sizeof(ut.session_id));
+			nextkn();
+			ut.nas_port = atoi(tok);
+			if (opttkn()) 
+				ut.framed_address = ipstr2long(tok);
 			printf("%d\n", checkrad(nas, &ut));
 			break;
 		case 'r':
@@ -1834,3 +1786,9 @@ test_shell()
 	}
 	return 0;
 }
+
+int
+master_process()
+{
+ 	return radius_pid == 0 || getpid() == radius_pid;
+}
