@@ -20,7 +20,7 @@
 #define RADIUS_MODULE 9
 #ifndef lint
 static char rcsid[] =
-"@(#) $Id: sql.c,v 1.11 2001/03/12 11:25:43 gray Exp $";
+"@(#) $Id: sql.c,v 1.12 2001/04/11 07:58:38 gray Exp $";
 #endif
 
 #if defined(HAVE_CONFIG_H)
@@ -488,7 +488,7 @@ rad_sql_init()
 	FREE(sql_cfg.acct_stop_query);
 	FREE(sql_cfg.acct_nasup_query);
 	FREE(sql_cfg.acct_nasdown_query);
-	FREE(new_cfg.acct_keepalive_query);
+	FREE(sql_cfg.acct_keepalive_query);
 	FREE(sql_cfg.buf.ptr);
 
 	/* copy new config */
