@@ -18,7 +18,7 @@
  */
 #ifndef lint
 static char rcsid[] = 
-"$Id: client.c,v 1.6 2001/03/02 16:54:02 gray Exp $";
+"$Id: client.c,v 1.7 2001/03/15 12:33:50 gray Exp $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -279,7 +279,8 @@ radclient_build_request(config, server, code, pair)
 				*ptr++ = attrlen + 2;
 				memset(passbuf, 0, attrlen);
 				memcpy(passbuf, pair->strvalue,
-				       pair->strlength);
+				       pair->strlength < AUTH_PASS_LEN ?
+                                              pair->strlength : AUTH_PASS_LEN);
 
 				/* Calculate the MD5 Digest */
 				secretlen = strlen(server->secret);
