@@ -26,7 +26,7 @@
 
 #ifndef lint
 static char rcsid[] =
- "@(#) $Id: postgres.c,v 1.4 2001/03/12 11:49:38 gray Exp $" ;
+ "@(#) $Id: postgres.c,v 1.5 2001/04/26 11:22:24 gray Exp $" ;
 #endif
 
 #include <stdlib.h>
@@ -240,6 +240,8 @@ rad_sql_query(conn, query, return_count)
 		radlog(L_ERR,
 		       ("PQexec returned %s"),
 		       PQresStatus(stat));
+		if (stat == PGRES_FATAL_ERROR) 
+			rad_sql_need_reconnect(conn->type);
 		rc = -1;
 	}
 	PQclear(res);
@@ -287,6 +289,8 @@ rad_sql_getpwd(conn, query)
 		radlog(L_ERR,
 		       _("PQexec returned %s"),
 		       PQresStatus(stat));
+		if (stat == PGRES_FATAL_ERROR) 
+			rad_sql_need_reconnect(conn->type);
 	}
 	PQclear(res);
 	return return_passwd;
@@ -332,6 +336,8 @@ rad_sql_exec(conn, query)
 		       _("PQexec returned %s"),
 		       PQresStatus(stat));
 		PQclear(res);
+		if (stat == PGRES_FATAL_ERROR) 
+			rad_sql_need_reconnect(conn->type);
 		return NULL;
 	}
 
