@@ -17,15 +17,17 @@
  *
  */
 
-static char rcsid[] = 
-"$Id: strerror.c,v 1.2 2000/12/29 11:39:59 gray Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #if !defined(HAVE_STRERROR)
 
+#ifndef lint
+static char rcsid[] = 
+"$Id: strerror.c,v 1.5 2001/03/12 11:49:38 gray Exp $";
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -41,7 +43,7 @@ strerror(err)
 	static char buf[80];
 
 	if (err > sys_nerr) {
-		sprintf(buf, "error %d", err);
+		radsprintf(buf, sizeof(buf), "error %d", err);
 		return buf;
 	}
 	return sys_errlist[err];
@@ -55,7 +57,7 @@ strerror(err)
 {
 	static char buf[80];
 
-	sprintf(buf, "error %d", err);
+	radsprintf(buf, sizeof(buf), "error %d", err);
 	return buf;
 }
 
